% ******Butterworthova aproximace******
% *******David Hradecky, 2003******
disp('Butterworthova aproximace');
n = ceil(log10(1/k1)/log10(1/k));   % Stupen filtru, zaokrouhleno nahoru na cele cislo
disp('Stupen filtru n :');
disp('=================');
n
% Prepocet parametru NDP na n cele
kn = k1^(1/n);          % Novy pomer meznich kmitoctu
OMEGASn = 1/kn;         % Novy hranickni kmitocet nepropustneho pasma
disp('Hranicni kmitocet nepropustneho pasma :');
OMEGASn
% Parametry standardniho tolerancniho schematu NDP
H_ap = 1/sqrt(1+epsilon^2);
H_as = 1/sqrt(1+epsilon^2/k1^2);
plot([0,OMEGAP,OMEGAP],[H_ap,H_ap,0],'b-',[OMEGASn,OMEGASn,OMEGASn+1],[1,H_as,H_as],'r-');
Title('Standardn tolerann schma NDP');
Xlabel('\Omega');
Ylabel('|H(j\Omega)|');
pause;
% Urceni polu prenosove fce H(s), (nulove body Butterworthova polynomu bn(s))
r = 1/epsilon^(1/n);    % Polomer kruznice s poly
for i = 1:1:n
    s(i) = r*(-sin((2*i-1)*pi/(2*n))+j*cos((2*i-1)*pi/(2*n)));  % Jen z leve poloroviny podle 6.24
    if abs(imag(s(i))) <= 0.00001   % Predpokladam jen zapornou imag. cast polu (stabilita)
        s(i) = real(s(i))+0*j;  % Nastaveni nuly imag. pri polu na realne ose a tak Matlabem vypoctene hodne male zbytecne hodnote
    end;
end
% Zobrazeni polu na kruznici v komplexni rovine
uhel = 0:0.01:6.28; 
for i = 1:1:629
    kruh(i) = r*exp(j*uhel(i)); % Vykresleni kruznice v kompl. rovine
end
S = zeros(2*n,1); % Pomocny vektor pro vykresleni spojnic polu s nulou
for i = 2:2:2*n
    S(i-1) = s(i/2);
end
plot([-r,r],[0,0],'k-',[0,0],[-r,r],'k-',real(S),imag(S),'c:',real(kruh),imag(kruh),'c:',real(s),imag(s),'r*');
axis([-(r+1/(10*r)),r+1/(10*r),-(r+1/(10*r)),r+1/(10*r)]);
%grid; % Mrizka
for i=1:1:n
    text(real(s(i)),imag(s(i))+0.08,num2str(s(i),4),'FontSize',8);  % Zobrazeni hodnot polu
end
Title('Hledan poly penosov funkce, s = \Sigma + j\Omega');
Xlabel('Real[s] = \Sigma');
Ylabel('Imag[s] = j\Omega');
pause;
% Vypocet prenosove funkce H(s) podle 6.26 a 6.27
alfa0 = -r;
alfa = real(s);
H0 = 1/epsilon;
if mod(n,2) == 1        % Je-li stupen filtru n lichy
    m = (n-1)/2;
    bn = [1,-alfa0];    % Repezentuje (s-alfa0) jmenovatele
else                    % Je-li n sudy
    m = n/2;
    bn = 1;
end
for i = 1:1:m   % Vypocet koeficientu bn(s) B. polynomu ve jmenovateli prenosove fce
    bp = [1,-2*alfa(i),alfa0^2];    % Zakladni polynom jmenovatele, vztah 6.27
    bn = conv(bn,bp);               % Soucin polynomu bn*bp jmenovatele pro n sude a liche
end                                 % Vysledne koef. jsou v bn
for i = 0:1:n % Kontrolni vypocet koef. tabulkovych pro epsilon=1 a a_max=3 dB
    bn_Tab(i+1) = bn(i+1)*epsilon^(i/n);    % Transformace podle 6.28
end
[H,w] = freqs(H0,bn);               % Automaticky vypocet hodnot prenosove fce H(s) pro nekolik omega podle 6.26,6.27
[Hn,wn] = freqs(1,bn_Tab);          % Normovana H(s), pro epsilon=1
[B,A] = butter(n,OMEGAP,'s');       % Vypocet prenosove fce s vyuzitim funkce Matlabu
[H_n,w_n] = freqs(B,A);             % Normovana H(s), epsilon=1
wo = 0:n/99:n;                      % Modulava charakteristika podle 6.10
Ho = sqrt(1./(1+wo.^(2*n).*epsilon^2));
Hb = tf(H0,bn);                     % Vysledna prenosova funkce ve tvaru funkce
%plot(w,abs(H),'b-',wn,abs(Hn),'r-',w_n,abs(H_n),'g-.',wo,Ho,'m-.'); %Srovnani modulovych charakteristik
%axis([0,n,0,1]);
%pause;
% Graf modulove charakteristiky + tolerancni sch. NDP
subplot(2,1,1);
plot(w,abs(H),'b-',[0,OMEGAP,OMEGAP],[H_ap,H_ap,0],'b:',[OMEGASn,OMEGASn,OMEGASn+1],[1,H_as,H_as],'r:');
axis([0,ceil(OMEGASn)+1,0,1]); % xmin,xmax,ymin,ymax
text(ceil((OMEGAP+(OMEGASn-OMEGAP)/2)*10)/10,0.8,['n = ',int2str(n)]); 
Title('Maximln ploch charakteristika modulu penosu NDP (Butterworth)');
Xlabel('\Omega');
Ylabel('|H(j\Omega)|');

epsilon % vypis zakladnich hodnot
H0
disp('Koeficienty Butterworthova polynomu pro epsilon<>1 a amax<>3 dB:');
bn
disp('Tabulkove koeficienty But. polynomu pro epsilon=1 a amax=3 dB:');
bn_Tab
disp('Prenosova fce H(s) NDP :');
disp('========================');
Hb
%pause;
% Vypocet charakteristicke funkce Fi(s)
Fi = [epsilon,zeros(1,n)];   % Umisteni na pozici v polynomu o stejne mocnine s^n pro dalsi pouziti v Zvs
Fi_Tab = [1,zeros(1,n)];     % epsilon=1
Fib = tf(Fi,1);              % Vysledna charakteristicka fce Fi(s) ve tvaru funkce
disp('Charakteristicka funkce Fi(s) :');
disp('===============================');
Fib
% Vypocet skupinoveho zpozdeni Tau
Tau_n = 0;
for i = 0:1:n-1
    Tau_n = Tau_n+wo.^(2*i)/sin((2*i+1)*pi/(2*n));  % Normovane skupinove zpozdeni podle 6.33
end
Tau_n = 1./(1+wo.^(2*n)).*Tau_n; % Normovane skupinove zpozdeni

% Vypocet skutecneho skupinoveho zpozdeni Tau
d_bn = polyder(bn); % derivovany But. polynom bn
[Tau,W] = freqs(d_bn,bn); % automaticke vycisleni zlomku d_bn(s)/bn(s) pro nejake kmitocty
Tau = real(Tau);    % Vypoctene skutecne skupinove zpozdeni podle definice 5.29, 6.33
subplot(2,1,2);
%plot(wo,Tau_n,'b-',W,Tau,'r-'); % jen pro porovnani skup. zpozdeni
plot(W,Tau,'b-');
axis([0,ceil(OMEGASn)+1,0,ceil(max(Tau))]); % xmin,xmax,ymin,ymax
text(ceil((OMEGAP+(OMEGASn-OMEGAP)/2)*10)/10,n-0.5,['n = ',int2str(n)]); 
Title('Charakteristika skupinovho zpodn NDP (Butterworth)');
Xlabel('\Omega');
Ylabel('\tau(\Omega)');
pause;
CLF;
% Fazova charakteristika
faze = angle(H);
subplot(2,1,1);
plot(w,(faze),'b-');
axis([0,ceil(OMEGASn)+1,-4,4]);
Xlabel('\Omega');
Ylabel('Faze');
Title('Fzov charakteristika NDP');
pause;

% ***** Realizace, urceni soucastek *****
% Vypocet pomoci fce sin pod vztahu 3.71 - lin.obvody , +Fi a r=1
for i = 1:1:n
    lc(i)=2*epsilon^(1/n)*sin((2*i-1)*pi/(2*n)); % Oba prvky, L1,C2,L3,...
end
% Vypocet rozkladem v retezovy zlomek (Cauer), odstepenim polu v nekonecnu
% Pro +Fi podle 5.39, r1=r2=1
% Kontrolni vypocet tabulkovych hodnot L a C NDP
z_ci = bn_Tab + Fi_Tab; % Staci pouzit Fi pro epsilon=1, protoze Ho=1/epsilon a to epsilon se vykrati pak vezlomku a zustane jen bn
z_jm = bn_Tab - Fi_Tab; % Pzn: zvs by sla vypocitat i pomoci transform. fci Hb a Fib a z nich pak zpetne ziskat cit. a jm. pomoci fce TFDATA
while z_jm(1) == 0 % Kdyz po odecteni bude prvni nula ve vektoru tak ji odstranim pred delenim
    z_jm = z_jm(1,2:end); % Vektor pak bude zacinat az od druhe pozice
end
[lc_Tab,r2] = Rozklad(z_ci,z_jm);  % Vlastni rozklad v retezovy zlomek pomoci fce Rozklad postupnym delenim
% Vypocet skutecnych hodnot
z_ci = bn + Fi_Tab; % Staci pouzit Fi pro epsilon=1, protoze Ho=1/epsilon a to epsilon se vykrati pak ve zlomku a zustane jen bn
z_jm = bn - Fi_Tab;
z_vs = tf(z_ci,z_jm);   % vstup. imp. transformovana do tvaru funkce
while z_jm(1) == 0 % Kdyz po odecteni bude prvni nula ve vektoru tak ji odstranim pred delenim
    z_jm = z_jm(1,2:end); % Vektor pak bude zacinat az od druhe pozice
end
[lcx,r2] = Rozklad(z_ci,z_jm);  % Vlastni rozklad v retezovy zlomek pomoci fce Rozklad postupnym delenim
r1 = 1;
disp(' Vstupni impedance pro +Fi : ');
disp('---------------------------- ');
z_vs
CLF;    % Smazu predchozi grafy
subplot(2,1,1);
Title('Schma filtru NDP, T lnek pro +Fi ');
Schema(lcx,r1,r2,n,'LC');   % Vykresleni schema zapojeni prickove struktury s prvnim L podelnym

% Pro -Fi podle 5.39, r1=r2=1
z_ci = bn - Fi_Tab; % Staci pouzit Fi pro epsilon=1, protoze Ho=1/epsilon a to epsilon se vykrati pak vezlomku a zustane jen bn
z_jm = bn + Fi_Tab;
z_vs = tf(z_ci,z_jm);   % vstup. imp. transformovana do tvaru funkce
while z_ci(1) == 0 % Kdyz po odecteni bude prvni nula ve vektoru tak ji odstranim pred delenim
    z_ci = z_ci(1,2:end); % Vektor pak bude zacinat az od druhe pozice
end
[clx,r2] = Rozklad(z_ci,z_jm); % Tady zaciname pricnym C1
disp(' Vstupni impedance pro -Fi : ');
disp('-----------------------------');
z_vs
subplot(2,1,2);
Title('Schma filtru NDP, \pi lnek pro -Fi ');
Schema(clx,r1,r2,n,'CL');   % Vykresleni schema zapojeni prickove struktury s prvnim C pricnym
pause;

% Vypocet prenosove funkce PP H(p)
Hb_p = Ftransf(Hb,d_omega,omega_m); % Transformace H(s) na H(p)
disp('Prenosova fce H(p) PP :');
disp('=======================');
Hb_p
[Hbp_ci,Hbp_jm] = tfdata(Hb_p,'v');  % Dostanu z transformovane fce zpet vektory citatele a jmenovatele
%[Hp,wp] = freqs(Hbp_ci,Hbp_jm);      % Automaticky vypocet hodnot prenosove fce H(p)
%fh = wp/(2*pi);
sf = ceil(f_s + (fs - f_s)/2);
fpoc = f_s - ceil(sf/5);
if fpoc < 0
   fpoc = 0;
end
fkon = fs + ceil(sf/5);
i = 1;
for f = fpoc+1:fkon/500:fkon    % Vhodne jen pro kmitocty blizke "nule", radove az kHz
    wp(i) = f*2*pi;
    fh(i) = f;
    i = i + 1;
end
Hp = freqs(Hbp_ci,Hbp_jm,wp);   % Vypocet pro vice omega nez to pocita automaticky, protoze je potreba vice hodnot pro H(p) vyssich radu
% Graf modulove charakteristiky H(p)+ tolerancni sch. PP
subplot(2,1,1);
plot(fh,abs(Hp),'b-',[0,f_s,f_s],[H_as,H_as,1],'r:',[f_p,f_p,fp,fp],[0,H_ap,H_ap,0],'b:',[fs,fs,fkon],[1,H_as,H_as],'r:');
axis([fpoc,fkon,0,1]); % xmin,xmax,ymin,ymax
Title('Maximln ploch charakteristika modulu penosu PP (Butterworth)');
Xlabel('f [Hz]');
Ylabel('|H(p)|');
% Vypocet skutecneho skupinoveho zpozdeni Tau
[d_G_ci,d_G_jm] = polyder(Hbp_jm,Hbp_ci); % derivovany podil dvou polynomu G(p)=Hbp_jm/Hbp_ci
Tci = conv(d_G_ci,Hbp_ci);    % d_G_ci*H_ci
Tjm = conv(d_G_jm,Hbp_jm);    % d_G_jm*H_jm
%[Tau,W] = freqs(Tci,Tjm);   % automaticke vycisleni zlomku dG(p)/G(p) pro nejake kmitocty
%fh = W/(2*pi);
Tau = freqs(Tci,Tjm,wp);   % Vypocet pro vice omega nez to pocita automaticky, pro vyssi rozliseni
Tau = abs(real(Tau));    % Vypoctene skutecne skupinove zpozdeni vychazejici z definice 5.29
subplot(2,1,2);
plot(fh,Tau,'b-');
axis([fpoc,fkon,0,max(Tau)+max(Tau)/10]); % xmin,xmax,ymin,ymax
Title('Charakteristika skupinovho zpodn PP (Butterworth)');
Xlabel('f [Hz]');
Ylabel('\tau [s]');
pause;
close all	% Zavreni oken s grafy (figure)

% Odnormovani
disp('-----------------------------------');
disp('* Zvolte impedancni uroven obvodu *');
disp('-----------------------------------');
R_0 = input(' Ro = '); % Impedancni uroven
%[LC] = Odnormuj(Soucastky,R0,dW,Wm,Stupen,Struktura)
[LCS] = Odnormuj(lcx,R_0,d_omega,omega_m,n,'LC');   % Odnormovani struktury T s prvnim podelnym L
CLF;    % Smazu predchozi
subplot(2,1,1);
Title(['Schma filtru T po odnormovn pro Ro = ',int2str(R_0),' \Omega']);
Schema(LCS,R_0,R_0,n,'LS');   % Vykresleni schema zapojeni prickove struktury s prvni LC seriove
[LCP] = Odnormuj(clx,R_0,d_omega,omega_m,n,'CL');   % Odnormovani struktury II s prvnim pricnym C
subplot(2,1,2);
Title(['Schma filtru \pi po odnormovn pro Ro = ',int2str(R_0),' \Omega']);
Schema(LCP,R_0,R_0,n,'LP');   % Vykresleni schema zapojeni prickove struktury s prvni LC paralelne

% Kaskadni synteza s OZ
disp(' ');
disp('******************************************');
disp('* Kaskadni synteza aktivniho filtru s OZ *');
disp('******************************************');
disp('* Volby :  1 - Ano                       *');
disp('*          2 - Ne                        *');
disp('******************************************');
synteza = input('Chcete ji provest ? : ');
switch synteza
case 1  
    Aktivni(Hb_p);
case 2

end
disp(' ');
disp('Konec syntezy filtru :o)');
disp('Autor : David Hradeck, 2003');